/*
  jQuery Form Generator Plugin
  Author: Sebastian Koch (js) & Markus Gramlich (css)
  Author URI: http://www.creanetsoft.de
  Version: 0.5

  Release history:

  0.5   - added trigger for refresh
  0.4   - changed event handling
        - added field trigger for clear and set
        - new function reset to clear form or to set to defined values
        - bug fixing: error text not hiding on reset
  0.3.1 - check box and radion button with new option checked: true/false
  0.3   - reset function added
  0.2   - change to object referencing
  0.1   - first version
  0.0.1 - first playing with plugin solution

  Todo:
  - submit on change - only sumbit form if a field has changed
  - interaction with plugin 'iframe-post-form'
  - !!! reset hidden elements !!!

  Bugs:
  - cleditor - value not shown

*/
(function($){
	var defaults =
	{
		id: "jFG"
		,enctype: "text/plain"
		,method: "POST"
		,action: "?"
		,buttons: []
		,hiddenElements: []
		,elements: []
		,
	};
	var methods = {
		// init and display one page
		init: function ( settings )
		{
			return this.each( function ()
			{
				var $this = $(this);
				var data = $this.data('jFormGenerator');

				if ( !data )
				{
					$this.data( 'jFormGenerator', $.extend( { target: $this }, defaults, settings ) );
					$.tmpl( jFG.templates.form, $this.data( 'jFormGenerator' )
						,{
							elementOptions: {
								storeData: function ( id, item )
								{
									$this.data('jFormGenerator')[ id ] = item;
								}
								,generateID: function ( type )
								{
									if ( !$(this)[0].counter[ type ] )
										$(this)[0].counter[ type ] = 0;
									return ( $(this)[0].counter[ type ]++ );
								}
								,getLastID: function ( type )
								{
									return ( $(this)[0].counter[ type ] - 1 );
								}
								,counter: {}
							}
						}
					).appendTo( $this.html('') ); // clear before insert !?

					$this.jFormGenerator( 'prepare' );
				}
			}).children('form');
		}
		,prepare: function ()
		{
			return this.each( function ()
			{
				var $this = $(this);
				var data = $this.data('jFormGenerator');

				if ( data )
				{
					$this.children('form').find(':input').not('[type=hidden]').not('button').not('[readonly]').each( function ()
					{
						$(this)
							.focus( function () { $this.jFormGenerator( 'elementFocus', this ) } )
							.blur( function () { $this.jFormGenerator( 'elementBlur', this ) } )
							.change( function () { $this.jFormGenerator( 'elementChanged', this ) } )
						;
						var dd = $(this).closest('dd');
						var id = $(dd).attr('jfg_id');
						if ( id )
						{
							var type = $this.data('jFormGenerator')[ id ].data.type;
							var loadFunction = jFG.load[ type ];
							if ( $.isFunction( loadFunction ) )
								loadFunction( $(this), $this.data('jFormGenerator')[ id ].data );

							// bind click event
							var clickFunction = $this.data('jFormGenerator')[ id ].data.click;
							if ( $.isFunction( clickFunction ) )
								$(this).bind( 'click', clickFunction );
							// bind events for clearing field
							var clearFunction = jFG.clear[ type ];
							if ( !$.isFunction( clearFunction ) )
								clearFunction = jFG.clear._default;
							$(this).bind( 'clear', clearFunction );
							// bind events for set field value
							var setFunction = jFG.set[ type ];
							if ( !$.isFunction( setFunction ) )
								setFunction = jFG.set._default;
							$(this).bind( 'set', setFunction );
							// bind events for refresh field
							var refreshFunction = jFG.refresh[ type ];
							if ( !$.isFunction( refreshFunction ) )
								refreshFunction = jFG.refresh._default;
							$(this).bind( 'refresh', refreshFunction );
						}
					});
					$this.children('form').find(':input[type=hidden]').each( function ()
					{
						// bind events for set field value
						var setFunction = jFG.set._default;
						$(this).bind( 'set', setFunction );
					});
					$this.children('form').find('button').each( function ()
					{
						var id = $(this).attr('jfg_id');
						if ( id )
						{
							// bind standard events
							var clickFunction = $this.data('jFormGenerator')[ id ].data.click;
							if ( $.isFunction( clickFunction ) )
								$(this).bind( 'click', clickFunction );
						}
					});
					$this.children('form').submit( function () { return $this.jFormGenerator( 'submit' ) } );
					/*?*/$this.children('form').find('button').button();
				}
			});
		}
		,reset: function ( values )
		{
			return this.each( function ()
			{
				var $this = $(this);
				var data = $this.data('jFormGenerator');

				if ( data )
				{
					// blur each field and clear value
					$this.find(':input').not('[readonly]').each( function ()
					{
						$(this).blur().trigger('clear');
						var dd = $(this).closest('dd');
						var error = $(dd).find('.error');
						$(error).text('');
					});
					$this.find('dd,dt').removeClass( function (index,classes)
					{
						//'focus fieldChanged'
						var matches = classes.match (/fieldError_.*|fieldError/g) || [];
						return (matches.join (' '));
					});
					for( item in values )
						$this.find(":input[name='" + item + "'],:input[name='" + item + "[]']").trigger( 'set', [values[item]] );
				}
			});
		}
		,refresh: function ()
		{
			return this.each( function ()
			{
				var $this = $(this);
				var data = $this.data('jFormGenerator');

				if ( data )
				{
					// blur each field and clear value
					$this.find(':input').not('[readonly]').trigger( 'refresh' );
				}
			});
		}
		,elementFocus: function ( element )
		{
			var $this = $(this);
			var data = $this.data('jFormGenerator');
			var dd = $(element).closest('dd');
			var id = $(dd).attr('jfg_id');
			if ( id )
			{
				var dt = $(dd).prev('dt');
				//var hint = $(dd).find('.hint');
				var type = $this.data('jFormGenerator')[ id ].data.type;
				//var hint_text = $this.data('jFormGenerator')[ id ].data.hint;

				$(dd).addClass('focus focus_'+type)
				$(dt).addClass('focus focus_'+type);
				//$(hint).text( hint_text );
			}
		}
		,elementBlur: function ( element )
		{
			var $this = $(this);
			var data = $this.data('jFormGenerator');
			var dd = $(element).closest('dd');
			var id = $(dd).attr('jfg_id');
			if ( id )
			{
				var dt = $(dd).prev('dt');
				var hint = $(dd).find('.hint');
				var type = $this.data('jFormGenerator')[ id ].data.type;
				var checkOnBlur = jFG.checkOnBlur[ type ];

				if ( checkOnBlur !== false )
					$this.jFormGenerator( 'elementCheck', element );
				$(dd).removeClass('focus focus_'+type)
				$(dt).removeClass('focus focus_'+type);
				//$(hint).text('');
			}
		}
		,elementCheck: function ( element )
		{
			var $this = $(this);
			var data = $this.data('jFormGenerator');
			var dd = $(element).closest('dd');
			var id = $(dd).attr('jfg_id');
			if ( id )
			{
				var dt = $(dd).prev('dt');
				var error = $(dd).find('.error');
				var type = $this.data('jFormGenerator')[ id ].data.type;
				var error_text = $this.data('jFormGenerator')[ id ].data.error;
				var checkFunction = $this.data('jFormGenerator')[ id ].data.check;

				if ( $.isFunction( checkFunction ) && !checkFunction( element, $(element).val() ) )
				{
					$(dd).addClass('fieldError fieldError_'+type)
					$(dt).addClass('fieldError fieldError_'+type);
					$(error).text( error_text );
				}
				else
				{
					$(dd).removeClass('fieldError fieldError_'+type)
					$(dt).removeClass('fieldError fieldError_'+type);
					$(error).text('');
				}
			}
		}
		,elementChanged: function ( element )
		{
			var $this = $(this);
			var data = $this.data('jFormGenerator');
			var dd = $(element).closest('dd');
			var id = $(dd).attr('jfg_id');
			if ( id )
			{
				var dt = $(dd).prev('dt');
				var type = $this.data('jFormGenerator')[ id ].data.type;

				$this.jFormGenerator( 'elementCheck', element );
				$(dd).addClass('fieldChanged fieldChanged_'+type)
				$(dt).addClass('fieldChanged fieldChanged_'+type);
			}
		}
		,formCheck: function ()
		{
			var $this = $(this);
			var data = $this.data('jFormGenerator');
			var form = $this.children('form');
			// check each field
			$this.children('form').find(':input').not('[readonly]').each( function () { $this.jFormGenerator( 'elementCheck', this ) } );
			// if no error check global checkFunction
			if ( $(form).find('dd.fieldError').length > 0 )
			{
				// error
				return false;
			}
			else
			{
				if ( $.isFunction( data.check ) && !data.check( form ) )
				{
					// error
					if ( data.error )
						application.Error(data.error);
					return false;
				}
				else
				{
					// happy
					return true;
				}
			}
		}
		,submit: function ()
		{
			var $this = $(this);
			var data = $this.data('jFormGenerator');
			var form = $this.children('form');

			if ( !$this.jFormGenerator( 'formCheck' ) )
			{
				// error
				return false;
			}
			else
			{
				if ( $.isFunction( data.submit ) )
				{
					data.submit( form );
					return false; // not do simple submit
				}
				else
				{
					return true; // simple submit
				}
			}
		}
	};

	$.fn.jFormGenerator = function ( method )
	{
		// Method calling logic
		if ( methods[ method ] )
			return methods[ method ].apply( this, Array.prototype.slice.call( arguments, 1 ) );
		else if ( typeof method === 'object' || ! method )
			return methods.init.apply( this, arguments );
		else
			$.error( 'Method ' +  method + ' does not exist on jQuery.jFormGenerator' );
	};
})(jQuery);

var jFG =
{
	templates:
	{
		form: '\
			<form enctype="${enctype}" method="${method}" action="${action}" id="${id}" class="jFG clearfix">\
				{{tmpl(hiddenElements) jFG.templates.hiddenElements}}\
				<dl>{{tmpl(elements,{elementOptions: $item.elementOptions}) jFG.templates.elements}}</dl>\
				<div class="jFG_buttons">{{tmpl(buttons) jFG.templates.buttons}}</div>\
			</form>\
		'
		,buttons: '\
			<!--${$item.parent.elementOptions.generateID(\'button\')}-->\
			<!--${$item.parent.elementOptions.storeData( "button_" + $item.parent.elementOptions.getLastID(\'button\'), $item )}-->\
			<button type="${type}" id="${id}" jfg_id="button_${$item.parent.elementOptions.getLastID(\'button\')}">${text}</button>\
		'
		,hiddenElements: '\
			<input type="hidden" name="${name}" value="${value}" jfg_id="hidden_${$item.parent.elementOptions.generateID(\'hiddenElements\')}">\
		'
		,elements: '\
				{{if fieldset}}\
					{{tmpl jFG.templates.fieldset}}\
				{{else}}\
					<!--${$item.elementOptions.generateID(\'elements\')}-->\
					<!--${$item.elementOptions.storeData( "element_" + $item.elementOptions.getLastID(\'elements\'), $item )}-->\
					{{tmpl jFG.templates.fields[type]}}\
				{{/if}}\
		'
		,fieldset: '\
			<fieldset class="${addClass}">\
				<legend>{{html fieldset}}</legend>\
				<dl>{{tmpl(elements,{elementOptions: $item.parent.elementOptions}) jFG.templates.elements}}</dl>\
			</fieldset>\
		'
		,fields:
		{
			label: '\
				<dt class="jFG_${type} ${addClass}"><label>${label}</label></dt>\
				<dd class="jFG_${type} ${addClass}" jfg_id="element_${$item.parent.elementOptions.getLastID(\'elements\')}">\
					<label>${value}</label>\
				</dd>\
			'
			,text: '\
				<dt class="jFG_${type} ${addClass}"><label>${label}</label></dt>\
				<dd class="jFG_${type} ${addClass}" jfg_id="element_${$item.parent.elementOptions.getLastID(\'elements\')}">\
					<input type="text" id="${id}" name="${name}" value="${value}" defaultValue="${defaultValue}">\
					{{tmpl jFG.templates.hint}}\
					{{tmpl jFG.templates.error}}\
				</dd>\
			'
			,textArea: '\
				<dt class="jFG_${type} ${addClass}"><label>${label}</label></dt>\
				<dd class="jFG_${type} ${addClass}" jfg_id="element_${$item.parent.elementOptions.getLastID(\'elements\')}">\
					<textarea id="${id}" name="${name}" defaultValue="${defaultValue}">${value}</textarea>\
					{{tmpl jFG.templates.hint}}\
					{{tmpl jFG.templates.error}}\
				</dd>\
			'
			,password: '\
				<dt class="jFG_${type} ${addClass}"><label>${label}</label></dt>\
				<dd class="jFG_${type} ${addClass}" jfg_id="element_${$item.parent.elementOptions.getLastID(\'elements\')}">\
					<input type="password" id="${id}" name="${name}" value="${value}">\
					{{tmpl jFG.templates.hint}}\
					{{tmpl jFG.templates.error}}\
				</dd>\
			'
			,file: '\
				<dt class="jFG_${type} ${addClass}"><label>${label}</label></dt>\
				<dd class="jFG_${type} ${addClass}" jfg_id="element_${$item.parent.elementOptions.getLastID(\'elements\')}">\
					<input type="file" id="${id}" name="${name}">\
					{{tmpl jFG.templates.hint}}\
					{{tmpl jFG.templates.error}}\
				</dd>\
			'
			,select: '\
				<dt class="jFG_${type} ${addClass}"><label>${label}</label></dt>\
				<dd class="jFG_${type} ${addClass}" jfg_id="element_${$item.parent.elementOptions.getLastID(\'elements\')}">\
					<select id="${id}" name="${name}">\
						{{tmpl(values) jFG.templates.fields.select_option}}\
					</select>\
					{{tmpl jFG.templates.hint}}\
					{{tmpl jFG.templates.error}}\
				</dd>\
			'
			,multiSelect: '\
				<dt class="jFG_${type} ${addClass}"><label>${label}</label></dt>\
				<dd class="jFG_${type} ${addClass}" jfg_id="element_${$item.parent.elementOptions.getLastID(\'elements\')}">\
					<select id="${id}" name="${name}[]" multiple size="${size}">\
						{{tmpl(values) jFG.templates.fields.select_option}}\
					</select>\
					{{tmpl jFG.templates.hint}}\
					{{tmpl jFG.templates.error}}\
				</dd>\
			'
			,select_group: '\
				<optgroup label="${group}">\
					{{tmpl(values) jFG.templates.fields.select_option}}\
				</optgroup>\
			'
			,select_option: '\
				{{if group}}\
					{{tmpl jFG.templates.fields.select_group}}\
				{{else}}\
					<option value="${value}">${title}</option>\
				{{/if}}\
			'
			,radio: '\
				<dt class="jFG_${type} ${addClass}">${text}</dt>\
				<dd class="jFG_${type} ${addClass}" jfg_id="element_${$item.parent.elementOptions.getLastID(\'elements\')}">\
					{{each(i,row) values}}\
						<span class="jFG_${type}_item"><input type="radio" id="${row.id}" name="${name}" value="${row.value}" ${row.checked ? "checked" : ""}><label>${row.label}</label></span>\
					{{/each}}\
					{{tmpl jFG.templates.hint}}\
					{{tmpl jFG.templates.error}}\
				</dd>\
			'
			,checkbox: '\
				<dt class="jFG_${type} ${addClass}">${text}</dt>\
				<dd class="jFG_${type} ${addClass}" jfg_id="element_${$item.parent.elementOptions.getLastID(\'elements\')}">\
					{{each(i,row) values}}\
						<span class="jFG_${type}_item"><input type="checkbox" id="${row.id}" name="${row.name}" value="${row.value}" ${row.checked ? "checked" : ""}><label>${row.label}</label></span>\
					{{/each}}\
					{{tmpl jFG.templates.hint}}\
					{{tmpl jFG.templates.error}}\
				</dd>\
			'
			,checkboxList: '\
				<dt class="jFG_${type} ${addClass}">${text}</dt>\
				<dd class="jFG_${type} ${addClass}" jfg_id="element_${$item.parent.elementOptions.getLastID(\'elements\')}">\
					{{each(i,row) values}}\
						<span class="jFG_${type}_item"><input type="checkbox" id="${row.id}" name="${name}[]" value="${row.value}" ${row.checked ? "checked" : ""}><label>${row.label}</label></span>\
					{{/each}}\
					{{tmpl jFG.templates.hint}}\
					{{tmpl jFG.templates.error}}\
				</dd>\
			'
			,richText: '\
				<dt class="jFG_${type} ${addClass}"><label>${label}</label></dt>\
				<dd class="jFG_${type} ${addClass}" jfg_id="element_${$item.parent.elementOptions.getLastID(\'elements\')}">\
					<textarea id="${id}" name="${name}" defaultValue="${defaultValue}">${value}</textarea>\
					{{tmpl jFG.templates.hint}}\
					{{tmpl jFG.templates.error}}\
				</dd>\
			'
			,datePicker: '\
				<dt class="jFG_${type} ${addClass}"><label>${label}</label></dt>\
				<dd class="jFG_${type} ${addClass}" jfg_id="element_${$item.parent.elementOptions.getLastID(\'elements\')}">\
					<input type="text" id="${id}" name="${name}" value="${value}" defaultValue="${defaultValue}">\
					{{tmpl jFG.templates.hint}}\
					{{tmpl jFG.templates.error}}\
				</dd>\
			'
		}
		,hint: '\
			{{if hint}}\
				<span class="hint">${hint}</span>\
			{{/if}}\
		'
		,error: '\
			{{if error}}\
				<span class="error"></span>\
			{{/if}}\
		'
	}
	,checkOnBlur:
	{
		datePicker: false
	}
	,load: // load function for each field type (optional)
	{
		richText: function ( field, data )
		{
			try
			{
				$(field).cleditor( data.options );
			}
			catch (e)
			{
				console.log('jQuery plugin "cleditor" is needed!');
			}
		}
		,datePicker: function ( field, data )
		{
			try
			{
				$(field).datepicker( data.options );
			}
			catch (e)
			{
				console.log('jQuery UI widget "Datepicker" is needed!');
			}
		}
	}
	,clear: // clear function for each field type (optional)
	{
		_default: function ()
		{
			$(this).val('');
		}
		,radio: function ()
		{
			$(this).attr('checked',false);
		}
		,checkbox: function ()
		{
			$(this).attr('checked',false);
		}
		,checkboxList: function ()
		{
			$(this).attr('checked',false);
		}
		,select: function ()
		{
			$(this).find('option:selected').attr('selected',false);
		}
		,multiSelect: function ()
		{
			$(this).find('option:selected').attr('selected',false);
		}
		,richText: function ()
		{
			$(this).val('');
			$(this).cleditor()[0].updateFrame();
		}
	}
	,set: // setValue function for each field type (optional)
	{
		_default: function ( event, value )
		{
			$(this).val( value );
		}
		,radio: function ( event, value )
		{
			$(this).attr( 'checked', $(this).val() == value );
		}
		,checkbox: function ( event, value )
		{
			$(this).attr( 'checked', $(this).val() == value );
		}
		,checkboxList: function ( event, value )
		{
			if ( typeof value == 'object' )
			{
				$(this).attr( 'checked', false );
				for( item in value )
					if ( $(this).val() == value[item] )
						$(this).attr( 'checked', true );
			}
			else
				$(this).attr( 'checked', $(this).val() == value );
		}
		,select: function ( event, value )
		{
			$(this).find('option:selected').attr( 'selected',false );
			$(this).find('option[value="' + value + '"]').attr( 'selected',true );
		}
		,multiSelect: function ( event, value )
		{
			$(this).find('option').each( function ()
			{
				if ( typeof value == 'object' )
				{
					$(this).attr( 'selected', false );
					for( item in value )
						if ( $(this).val() == value[item] )
							$(this).attr( 'selected', true );
				}
				else
					$(this).attr( 'selected', $(this).val() == value );
			});
		}
		,richText: function ( event, value )
		{
			$(this).val( value );
			$(this).cleditor()[0].updateFrame();
		}
	}
	,refresh:
	{
		_default: function () {}
		,richText: function ()
		{
			$(this).cleditor()[0].refresh();
		}
	}
}


